@@ -1,3 +1,4 @@
+# ks 21-12-09: Func <- unlist() ... output variables now set in C-code
 
 ### ============================================================================
 ### lsodar -- solves ordinary differential equation systems
@@ -23,7 +24,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   maxordn = 12, maxords = 5, bandup = NULL, banddown = NULL, 
   maxsteps = 5000, dllname=NULL,initfunc=dllname, initpar=parms,
   rpar=NULL, ipar=NULL, nout=0, outnames=NULL, forcings=NULL,
-  initforc = NULL, fcontrol=NULL, events=NULL, ...)    {
+  initforc = NULL, fcontrol=NULL, events=NULL, lags = NULL, ...)    {
 
 ### check input
   hmax <- checkInput (y, times, func, rtol, atol,
@@ -108,7 +109,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     if (ynames)    {
       Func    <- function(time,state) {
         attr(state,"names") <- Ynames
-        func   (time,state,parms,...)[1]
+         unlist(func   (time,state,parms,...))
       }
       Func2   <- function(time,state)  {
         attr(state,"names") <- Ynames
@@ -130,7 +131,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
          }
     } else {                            # no ynames...
       Func    <- function(time,state)
-        func   (time,state,parms,...)[1]
+         unlist(func   (time,state,parms,...))
         
       Func2   <- function(time,state)
         func   (time,state,parms,...)
@@ -164,6 +165,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
         stop("root function 'rootfunc' must return a vector\n")
       nroot <- length(tmp2)
     } else nroot = 0
+    
     if (jt %in% c(1,4)) {
       tmp <- eval(JacFunc(times[1], y), rho)
       if (!is.matrix(tmp))
@@ -196,6 +198,7 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   iwork[6] <- maxsteps
   if (maxordn != 12) iwork[8] <- maxordn
   if (maxords != 5)  iwork[9] <- maxords
+  if (verbose)  iwork[5] = 1    # prints method switches to screen
 
   if(! is.null(tcrit)) rwork[1] <- tcrit
   rwork[5] <- hini
@@ -214,13 +217,16 @@ lsodar <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
 ### calling solver    
   storage.mode(y) <- storage.mode(times) <- "double"
   IN <-4
+
+  lags <- checklags(lags) 
+
   out <- .Call("call_lsoda",y,times,Func,initpar,
                rtol, atol, rho, tcrit, JacFunc, ModelInit, Eventfunc,
                as.integer(verbose), as.integer(itask), as.double(rwork),
                as.integer(iwork), as.integer(jt),as.integer(Nglobal),
                as.integer(lrw),as.integer(liw),as.integer(IN),RootFunc,
                as.integer(nroot), as.double (rpar), as.integer(ipar),
-               as.integer(0), flist, events, PACKAGE="deSolve")   
+               as.integer(0), flist, events, lags, PACKAGE="deSolve")   
 
 ### saving results    
   iroot  <- attr(out, "iroot")
